@@ -1,5 +1,5 @@
 /* data.h - Internal data object abstraction interface.
-   Copyright (C) 2002, 2004 g10 Code GmbH
+   Copyright (C) 2002, 2004, 2005 g10 Code GmbH
  
    This file is part of GPGME.
  
@@ -77,6 +77,9 @@ struct gpgme_data
   char pending[BUFFER_SIZE];
   int pending_len;
 
+  /* File name of the data object.  */
+  char *file_name;
+
   union
   {
     /* For gpgme_data_new_from_fd.  */
