@@ -1,6 +1,6 @@
 /* posix-io.c - Posix I/O functions
    Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2004, 2005, 2007 g10 Code GmbH
+   Copyright (C) 2001, 2002, 2004, 2005 g10 Code GmbH
 
    This file is part of GPGME.
  
@@ -33,7 +33,6 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <ctype.h>
 
 #include "util.h"
 #include "priv-io.h"
@@ -70,44 +69,48 @@ _gpgme_io_fd2str (char *buf, int buflen, int fd)
 
 static struct
 {
-  _gpgme_close_notify_handler_t handler;
+  void (*handler) (int,void*);
   void *value;
 } notify_table[256];
 
-
 int
 _gpgme_io_read (int fd, void *buffer, size_t count)
 {
   int nread;
-  TRACE_BEG2 (DEBUG_SYSIO, "_gpgme_io_read", fd,
-	      "buffer=%p, count=%u", buffer, count);
+  int saved_errno;
 
+  DEBUG2 ("fd %d: about to read %d bytes\n", fd, (int) count);
   do
     {
       nread = _gpgme_ath_read (fd, buffer, count);
     }
   while (nread == -1 && errno == EINTR);
-
-  TRACE_LOGBUF (buffer, nread);
-  return TRACE_SYSRES (nread);
+  saved_errno = errno;
+  DEBUG2 ("fd %d: got %d bytes\n", fd, nread);
+  if (nread > 0)
+    _gpgme_debug (2, "fd %d: got `%.*s'\n", fd, nread, buffer);
+  errno = saved_errno;
+  return nread;
 }
 
 
 int
 _gpgme_io_write (int fd, const void *buffer, size_t count)
 {
+  int saved_errno;
   int nwritten;
-  TRACE_BEG2 (DEBUG_SYSIO, "_gpgme_io_write", fd,
-	      "buffer=%p, count=%u", buffer, count);
-  TRACE_LOGBUF (buffer, count);
 
+  DEBUG2 ("fd %d: about to write %d bytes\n", fd, (int) count);
+  _gpgme_debug (2, "fd %d: write `%.*s'\n", fd, (int) count, buffer);
   do
     {
       nwritten = _gpgme_ath_write (fd, buffer, count);
     }
   while (nwritten == -1 && errno == EINTR);
-
-  return TRACE_SYSRES (nwritten);
+  saved_errno = errno;
+  DEBUG2 ("fd %d:          wrote %d bytes\n", fd, (int) nwritten);
+  errno = saved_errno;
+  return nwritten;
 }
 
 
@@ -116,14 +119,10 @@ _gpgme_io_pipe (int filedes[2], int inherit_idx)
 {
   int saved_errno;
   int err;
-  TRACE_BEG2 (DEBUG_SYSIO, "_gpgme_io_pipe", filedes,
-	      "inherit_idx=%i (GPGME uses it for %s)",
-	      inherit_idx, inherit_idx ? "reading" : "writing");
 
   err = pipe (filedes);
   if (err < 0)
-    return TRACE_SYSRES (err);
-
+    return err;
   /* FIXME: Should get the old flags first.  */
   err = fcntl (filedes[1 - inherit_idx], F_SETFD, FD_CLOEXEC);
   saved_errno = errno;
@@ -133,61 +132,42 @@ _gpgme_io_pipe (int filedes[2], int inherit_idx)
       close (filedes[1]);
     }
   errno = saved_errno;
-  if (err)
-    return TRACE_SYSRES (err);
-
-  return TRACE_SUC2 ("read=0x%x, write=0x%x", filedes[0], filedes[1]);
+  return err;
 }
 
 
 int
 _gpgme_io_close (int fd)
 {
-  int res;
-
-  TRACE_BEG (DEBUG_SYSIO, "_gpgme_io_close", fd);
-
   if (fd == -1)
-    {
-      errno = EINVAL;
-      return TRACE_SYSRES (-1);
-    }
-
+    return -1;
   /* First call the notify handler.  */
+  DEBUG1 ("closing fd %d", fd);
   if (fd >= 0 && fd < (int) DIM (notify_table))
     {
       if (notify_table[fd].handler)
 	{
-	  TRACE_LOG2 ("invoking close handler %p/%p",
-		      notify_table[fd].handler, notify_table[fd].value);
 	  notify_table[fd].handler (fd, notify_table[fd].value);
 	  notify_table[fd].handler = NULL;
 	  notify_table[fd].value = NULL;
         }
     }
   /* Then do the close.  */    
-  res = close (fd);
-  return TRACE_SYSRES (res);
+  return close (fd);
 }
 
 
 int
-_gpgme_io_set_close_notify (int fd, _gpgme_close_notify_handler_t handler,
-			    void *value)
+_gpgme_io_set_close_notify (int fd, void (*handler)(int, void*), void *value)
 {
-  TRACE_BEG2 (DEBUG_SYSIO, "_gpgme_io_set_close_notify", fd,
-	      "close_handler=%p/%p", handler, value);
-
   assert (fd != -1);
 
   if (fd < 0 || fd >= (int) DIM (notify_table))
-    {
-      errno = EINVAL;
-      return TRACE_SYSRES (-1);
-    }
+    return -1;
+  DEBUG1 ("set notification for fd %d", fd);
   notify_table[fd].handler = handler;
   notify_table[fd].value = value;
-  return TRACE_SYSRES (0);
+  return 0;
 }
 
 
@@ -195,15 +175,12 @@ int
 _gpgme_io_set_nonblocking (int fd)
 {
   int flags;
-  int res;
-  TRACE_BEG (DEBUG_SYSIO, "_gpgme_io_set_nonblocking", fd);
 
   flags = fcntl (fd, F_GETFL, 0);
   if (flags == -1)
-    return TRACE_SYSRES (-1);
+    return -1;
   flags |= O_NONBLOCK;
-  res = fcntl (fd, F_SETFL, flags);
-  return TRACE_SYSRES (res);
+  return fcntl (fd, F_SETFL, flags);
 }
 
 
@@ -239,21 +216,12 @@ _gpgme_io_spawn (const char *path, char **argv,
 {
   pid_t pid;
   int i;
-  int status;
-  int signo;
-  TRACE_BEG1 (DEBUG_SYSIO, "_gpgme_io_spawn", path,
-	      "path=%s", path);
-  i = 0;
-  while (argv[i])
-    {
-      TRACE_LOG2 ("argv[%2i] = %s", i, argv[i]);
-      i++;
-    }
-  
+  int status, signo;
+
   pid = fork ();
   if (pid == -1) 
-    return TRACE_SYSRES (-1);
-  
+    return -1;
+
   if (!pid)
     {
       /* Intermediate child to prevent zombie processes.  */
@@ -276,12 +244,7 @@ _gpgme_io_spawn (const char *path, char **argv,
 		  if (dup2 (fd_child_list[i].fd,
 			    fd_child_list[i].dup_to) == -1)
 		    {
-#if 0
-		      /* FIXME: The debug file descriptor is not
-			 dup'ed anyway, so we can't see this.  */
-		      TRACE_LOG1 ("dup2 failed in child: %s\n",
-				  strerror (errno));
-#endif
+		      DEBUG1 ("dup2 failed in child: %s\n", strerror (errno));
 		      _exit (8);
 		    }
 		  if (fd_child_list[i].dup_to == 0)
@@ -297,12 +260,7 @@ _gpgme_io_spawn (const char *path, char **argv,
 	      int fd = open ("/dev/null", O_RDWR);
 	      if (fd == -1)
 		{
-#if 0
-		  /* FIXME: The debug file descriptor is not dup'ed
-		     anyway, so we can't see this.  */
-		  TRACE_LOG1 ("can't open `/dev/null': %s\n",
-			      strerror (errno));
-#endif
+		  DEBUG1 ("can't open `/dev/null': %s\n", strerror (errno));
 		  _exit (8);
 		}
 	      /* Make sure that the process has a connected stdin.  */
@@ -310,24 +268,16 @@ _gpgme_io_spawn (const char *path, char **argv,
 		{
 		  if (dup2 (fd, 0) == -1)
 		    {
-#if 0
-		  /* FIXME: The debug file descriptor is not dup'ed
-		     anyway, so we can't see this.  */
-		      TRACE_LOG1 ("dup2(/dev/null, 0) failed: %s\n",
-				  strerror (errno));
-#endif
+		      DEBUG1("dup2(/dev/null, 0) failed: %s\n",
+			     strerror (errno));
 		      _exit (8);
 		    }
 		}
 	      if (!duped_stderr)
 		if (dup2 (fd, 2) == -1)
 		  {
-#if 0
-		    /* FIXME: The debug file descriptor is not dup'ed
-		       anyway, so we can't see this.  */
-		    TRACE_LOG1 ("dup2(dev/null, 2) failed: %s\n",
-				strerror (errno));
-#endif
+		    DEBUG1 ("dup2(dev/null, 2) failed: %s\n",
+			    strerror (errno));
 		    _exit (8);
 		  }
 	      close (fd);
@@ -336,11 +286,7 @@ _gpgme_io_spawn (const char *path, char **argv,
 	  execv ( path, argv );
 	  /* Hmm: in that case we could write a special status code to the
 	     status-pipe.  */
-#if 0
-	  /* FIXME: The debug file descriptor is not dup'ed anyway, so
-	     we can't see this.  */
-	  TRACE_LOG1 ("exec of `%s' failed\n", path);
-#endif
+	  DEBUG1 ("exec of `%s' failed\n", path);
 	  _exit (8);
 	} /* End child.  */
       if (pid == -1)
@@ -348,37 +294,34 @@ _gpgme_io_spawn (const char *path, char **argv,
       else
 	_exit (0);
     }
-
-  TRACE_LOG1 ("waiting for child process pid=%i", pid);
+    
   _gpgme_io_waitpid (pid, 1, &status, &signo);
   if (status)
-    return TRACE_SYSRES (-1);
+    return -1;
 
   /* .dup_to is not used in the parent list.  */
   for (i = 0; fd_parent_list[i].fd != -1; i++)
     _gpgme_io_close (fd_parent_list[i].fd);
 
-  return TRACE_SYSRES (0);
+  return 0;
 }
 
 
-/* Select on the list of fds.  Returns: -1 = error, 0 = timeout or
-   nothing to select, > 0 = number of signaled fds.  */
+/*
+ * Select on the list of fds.
+ * Returns: -1 = error
+ *           0 = timeout or nothing to select
+ *          >0 = number of signaled fds
+ */
 int
 _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
 {
   fd_set readfds;
   fd_set writefds;
   unsigned int i;
-  int any;
-  int max_fd;
-  int n;
-  int count;
-  /* Use a 1s timeout.  */
-  struct timeval timeout = { 1, 0 };
+  int any, max_fd, n, count;
+  struct timeval timeout = { 1, 0 }; /* Use a 1s timeout.  */
   void *dbg_help = NULL;
-  TRACE_BEG2 (DEBUG_SYSIO, "_gpgme_io_select", fds,
-	      "nfds=%u, nonblock=%u", nfds, nonblock);
 
   FD_ZERO (&readfds);
   FD_ZERO (&writefds);
@@ -386,20 +329,21 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
   if (nonblock)
     timeout.tv_sec = 0;
 
-  TRACE_SEQ (dbg_help, "select on [ ");
-
+  DEBUG_BEGIN (dbg_help, 3, "gpgme:select on [ ");
   any = 0;
   for (i = 0; i < nfds; i++)
     {
       if (fds[i].fd == -1) 
 	continue;
-      if (fds[i].for_read)
+      if (fds[i].frozen)
+	DEBUG_ADD1 (dbg_help, "f%d ", fds[i].fd);
+      else if (fds[i].for_read)
 	{
 	  assert (!FD_ISSET (fds[i].fd, &readfds));
 	  FD_SET (fds[i].fd, &readfds);
 	  if (fds[i].fd > max_fd)
 	    max_fd = fds[i].fd;
-	  TRACE_ADD1 (dbg_help, "r0x%x ", fds[i].fd);
+	  DEBUG_ADD1 (dbg_help, "r%d ", fds[i].fd);
 	  any = 1;
         }
       else if (fds[i].for_write)
@@ -408,14 +352,14 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
 	  FD_SET (fds[i].fd, &writefds);
 	  if (fds[i].fd > max_fd)
 	    max_fd = fds[i].fd;
-	  TRACE_ADD1 (dbg_help, "w0x%x ", fds[i].fd);
+	  DEBUG_ADD1 (dbg_help, "w%d ", fds[i].fd);
 	  any = 1;
         }
       fds[i].signaled = 0;
     }
-  TRACE_END (dbg_help, "]"); 
+  DEBUG_END (dbg_help, "]"); 
   if (!any)
-    return TRACE_SYSRES (0);
+    return 0;
 
   do
     {
@@ -424,22 +368,27 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
     }
   while (count < 0 && errno == EINTR);
   if (count < 0)
-    return TRACE_SYSRES (-1);
+    {
+      int saved_errno = errno;
+      DEBUG1 ("_gpgme_io_select failed: %s\n", strerror (errno));
+      errno = saved_errno;
+      return -1; /* error */
+    }
 
-  TRACE_SEQ (dbg_help, "select OK [ ");
-  if (TRACE_ENABLED (dbg_help))
+  DEBUG_BEGIN (dbg_help, 3, "select OK [ ");
+  if (DEBUG_ENABLED (dbg_help))
     {
       for (i = 0; i <= max_fd; i++)
 	{
 	  if (FD_ISSET (i, &readfds))
-	    TRACE_ADD1 (dbg_help, "r0x%x ", i);
+	    DEBUG_ADD1 (dbg_help, "r%d ", i);
 	  if (FD_ISSET (i, &writefds))
-	    TRACE_ADD1 (dbg_help, "w0x%x ", i);
+	    DEBUG_ADD1 (dbg_help, "w%d ", i);
         }
-      TRACE_END (dbg_help, "]");
+      DEBUG_END (dbg_help, "]");
     }
     
-  /* The variable N is used to optimize it a little bit.  */
+  /* n is used to optimize it a little bit.  */
   for (n = count, i = 0; i < nfds && n; i++)
     {
       if (fds[i].fd == -1)
@@ -461,90 +410,5 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
             }
         }
     }
-  return TRACE_SYSRES (count);
-}
-
-
-int
-_gpgme_io_recvmsg (int fd, struct msghdr *msg, int flags)
-{
-  int nread;
-  int saved_errno;
-  struct iovec *iov;
-  TRACE_BEG2 (DEBUG_SYSIO, "_gpgme_io_recvmsg", fd,
-	      "msg=%p, flags=%i", msg, flags);
-
-  nread = 0;
-  iov = msg->msg_iov;
-  while (iov < msg->msg_iov + msg->msg_iovlen)
-    {
-      nread += iov->iov_len;
-      iov++;
-    }
-  
-  TRACE_LOG1 ("about to receive %d bytes", nread);
-
-  do
-    {
-      nread = _gpgme_ath_recvmsg (fd, msg, flags);
-    }
-  while (nread == -1 && errno == EINTR);
-  saved_errno = errno;
-  if (nread > 0)
-    {
-      int nr = nread;
-
-      iov = msg->msg_iov;
-      while (nr > 0)
-	{
-	  int len = nr > iov->iov_len ? iov->iov_len : nr;
-	  TRACE_LOGBUF (msg->msg_iov->iov_base, len);
-	  iov++;
-	  nr -= len;
-	}
-    }
-  errno = saved_errno;
-  return TRACE_SYSRES (nread);
-}
-
-
-int
-_gpgme_io_sendmsg (int fd, const struct msghdr *msg, int flags)
-{
-  int nwritten;
-  struct iovec *iov;
-  TRACE_BEG2 (DEBUG_SYSIO, "_gpgme_io_sendmsg", fd,
-	      "msg=%p, flags=%i", msg, flags);
-
-  nwritten = 0;
-  iov = msg->msg_iov;
-  while (iov < msg->msg_iov + msg->msg_iovlen)
-    {
-      nwritten += iov->iov_len;
-      iov++;
-    }
-
-  TRACE_LOG1 ("about to receive %d bytes", nwritten);
-  iov = msg->msg_iov;
-  while (nwritten > 0)
-    {
-      int len = nwritten > iov->iov_len ? iov->iov_len : nwritten;
-      TRACE_LOGBUF (msg->msg_iov->iov_base, len);
-      iov++;
-      nwritten -= len;
-    }
-
-  do
-    {
-      nwritten = _gpgme_ath_sendmsg (fd, msg, flags);
-    }
-  while (nwritten == -1 && errno == EINTR);
-  return TRACE_SYSRES (nwritten);
-}
-
-
-int
-_gpgme_io_dup (int fd)
-{
-  return dup (fd);
+  return count;
 }
